@@ -1,3 +1,7 @@
+from __future__ import division
+from __future__ import print_function
+from builtins import zip
+from past.utils import old_div
 import argparse
 import logging
 import time
@@ -54,8 +58,8 @@ class MeasurementDriver(DriverBase):
       m = Machine(name=hostname,
                   cpu=_cputype(),
                   cores=_cpucount(),
-                  memory_gb=_memorysize() / (
-                  1024.0 ** 3) if _memorysize() else 0,
+                  memory_gb=old_div(_memorysize(), (
+                  1024.0 ** 3)) if _memorysize() else 0,
                   machine_class=self.get_machine_class())
       self.session.add(m)
       return m
@@ -121,9 +125,13 @@ class MeasurementDriver(DriverBase):
 
     self.input_manager.before_run(desired_result, input)
 
-    result = self.interface.run_precompiled(desired_result, input,
-                                            desired_result.limit,
-                                            compile_result, exec_id)
+    if self.interface.parallel_compile:
+        result = self.interface.run_precompiled(desired_result, input,
+                                                desired_result.limit,
+                                                compile_result, exec_id)
+    else:
+        result = self.interface.compile_and_run(desired_result, input,
+                                                desired_result.limit)
 
     self.report_result(desired_result, result, input)
 
@@ -178,6 +186,8 @@ class MeasurementDriver(DriverBase):
         if self.claim_desired_result(dr):
           desired_results.append(dr)
           thread_args.append((self.interface, dr.configuration.data, dr.id))
+      if len(desired_results) == 0:
+        return
       thread_pool = ThreadPool(len(desired_results))
       # print 'Compiling %d results' % len(thread_args)
       try:
@@ -196,9 +206,10 @@ class MeasurementDriver(DriverBase):
         self.run_desired_result(dr, compile_result, dr.id)
         try:
           self.interface.cleanup(dr.id)
-        except RuntimeError, e:
-          print e
+        except RuntimeError as e:
+          print(e)
           # print 'Done!'
+      thread_pool.close()
     else:
       for dr in q.all():
         if self.claim_desired_result(dr):
