@@ -1,3 +1,5 @@
+from builtins import range
+from builtins import object
 import abc
 import argparse
 import errno
@@ -10,6 +12,7 @@ import subprocess
 import threading
 import time
 from multiprocessing.pool import ThreadPool
+from future.utils import with_metaclass
 
 try:
   import resource
@@ -23,21 +26,22 @@ except ImportError:
 
 import opentuner
 from opentuner import resultsdb
+from opentuner.resultsdb.models import *
 
 log = logging.getLogger(__name__)
 
 argparser = argparse.ArgumentParser(add_help=False)
 argparser.add_argument('--parallel-compile', action='store_true',
+                       default=False,
                        help="present if compiling can be done in parallel")
 
 the_io_thread_pool = None
 
 
-class MeasurementInterface(object):
+class MeasurementInterface(with_metaclass(abc.ABCMeta, object)):
   """
   abstract base class for compile and measurement
   """
-  __metaclass__ = abc.ABCMeta
 
   def __init__(self,
                args=None,
@@ -57,26 +61,36 @@ class MeasurementInterface(object):
 
     self.pids = []
     self.pid_lock = threading.Lock()
-    self.parallel_compile = False
+    self.parallel_compile = args.parallel_compile
+    # If parallel_compile is False then compile_and_run() will be invoked
+    # sequentially otherwise the driver first invokes compile() in parallel
+    # followed by run_precompiled() sequentially
 
   def compile(self, config_data, id):
     """
-    Compiles according to the configuration in config_data (obtained from
-    desired_result.configuration) Should use id paramater to determine output
-    location of executable Return value will be passed to run_precompiled
-    as compile_result, useful for storing error/timeout information
+    Compile in PARALLEL according to the configuration in config_data 
+    (obtained from desired_result.configuration) Should use id parameter 
+    to determine output location of executable Return value will be passed 
+    to run_precompiled as compile_result, useful for storing error/timeout 
+    information
     """
+    if self.parallel_compile:
+        raise RuntimeError('MeasurementInterface.compile() not implemented for',
+                'parallel compilation')
     pass
 
   def run_precompiled(self, desired_result, input, limit, compile_result, id):
     """
-    Runs the given desired result on input and produce a Result() Abort
-    early if limit (in seconds) is reached Assumes that the executable
-    to be measured is already compiled in an executable corresponding to
-    identifier id compile_result is the return result of compile(), will be
-    None if compile was not called If id = None, must call run()
+    Run the given desired_result SEQUENTIALLY on input and produce a Result() 
+    Abort early if limit (in seconds) is reached Assume that the executable
+    to be measured has already been compiled to an executable corresponding to
+    identifier id by compile() The compile_result is the return result of compile(), 
+    and it will be None if compile() was not called
     """
-    return self.run(desired_result, input, limit)
+    if self.parallel_compile:
+        raise RuntimeError('MeasurementInterface.run_precompiled() not implemented', 
+                'for parallel compilation')
+    pass
 
   def cleanup(self, id):
     """
@@ -84,7 +98,39 @@ class MeasurementInterface(object):
     """
     pass
 
-  @abc.abstractmethod
+  def pre_process(self):
+    """
+    The process before each iteration This method will be called
+    once per iteration before all threads are launched
+    """
+    pass
+
+  def post_process(self):
+    """
+    The process after each iteration This method will be called 
+    once per iteration after all threads are committed
+    """
+    pass
+
+  def extra_convergence_criteria(self, result):
+    """
+    The extra convergence criteria which returns True if the
+    current result is acceptable by the user
+    """
+    return False
+
+  #@abc.abstractmethod
+  def compile_and_run(self, desired_result, input, limit):
+    """
+    Compile and run the given desired_result on input and produce a 
+    Result(), abort early if limit (in seconds) is reached This function 
+    is only used for sequential execution flow
+
+    FIXME: Shoud uncomment @abc.abstractmethod Now comment out for
+    compatiability
+    """
+    return self.run(desired_result, input, limit)
+
   def run(self, desired_result, input, limit):
     """
     run the given desired_result on input and produce a Result(),
@@ -112,6 +158,7 @@ class MeasurementInterface(object):
         project=self.project_name(),
         name=self.program_name(),
         version=self.program_version(),
+        parameter_info=self.manipulator().parameters_to_json(),
     )
 
   def set_driver(self, measurement_driver):
@@ -193,7 +240,7 @@ class MeasurementInterface(object):
     the_io_thread_pool_init(self.args.parallelism)
     if limit is float('inf'):
       limit = None
-    if type(cmd) in (str, unicode):
+    if type(cmd) in (str, str):
       kwargs['shell'] = True
     killed = False
     t0 = time.time()
@@ -282,7 +329,7 @@ def the_io_thread_pool_init(parallelism=1):
   if the_io_thread_pool is None:
     the_io_thread_pool = ThreadPool(2 * parallelism)
     # make sure the threads are started up
-    the_io_thread_pool.map(int, range(2 * parallelism))
+    the_io_thread_pool.map(int, list(range(2 * parallelism)))
 
 
 def goodkillpg(pid):
@@ -307,7 +354,7 @@ def goodwait(p):
     try:
       rv = p.wait()
       return rv
-    except OSError, e:
+    except OSError as e:
       if e.errno != errno.EINTR:
         raise
 
