@@ -1,7 +1,11 @@
+from __future__ import print_function
+from builtins import range
+from builtins import object
 import argparse
 import copy
 import logging
 import os
+import sys
 
 from datetime import datetime
 from fn import _
@@ -9,13 +13,16 @@ from opentuner.driverbase import DriverBase
 from opentuner.resultsdb.models import Configuration
 from opentuner.resultsdb.models import DesiredResult
 from opentuner.resultsdb.models import Result
+from opentuner.resultsdb.models import BanditInfo
+from opentuner.resultsdb.models import BanditSubTechnique
 from opentuner.search import plugin
 from opentuner.search import technique
+from opentuner.search.bandittechniques import AUCBanditMetaTechnique
 
 log = logging.getLogger(__name__)
 
 argparser = argparse.ArgumentParser(add_help=False)
-argparser.add_argument('--test-limit', type=int, default=5000,
+argparser.add_argument('--test-limit', type=int,
                        help='stop tuning after given tests count')
 argparser.add_argument('--stop-after', type=float,
                        help='stop tuning after given seconds')
@@ -41,23 +48,48 @@ class SearchDriver(DriverBase):
   DesiredResults
   """
 
-  def __init__(self, manipulator, extra_seeds=None, **kwargs):
+  def __init__(self, manipulator, extra_seeds=None, extra_criteria=None, **kwargs):
     super(SearchDriver, self).__init__(**kwargs)
     if extra_seeds is None:
       extra_seeds = []
     self.manipulator = manipulator
     self.wait_for_results = self.tuning_run_main.results_wait
     self.commit = self.tuning_run_main.commit
+    self.extra_criteria = extra_criteria
 
     self.generation = 0
     self.test_count = 0
     self.plugins = plugin.get_enabled(self.args)
     self.pending_result_callbacks = list()  # (DesiredResult, function) tuples
     # deepcopy is required to have multiple tuning runs in a single process
-    self.root_technique = copy.deepcopy(technique.get_root(self.args))
+    if self.args.list_techniques:
+      techniques, generators = technique.all_techniques()
+      for t in techniques:
+        print(t.name)
+      sys.exit(0)
+
+    if self.args.generate_bandit_technique:
+      # generate a bandit
+      self.root_technique = AUCBanditMetaTechnique.generate_technique(manipulator)
+    else:
+      self.root_technique = copy.deepcopy(technique.get_root(self.args))
+
+    if isinstance(self.root_technique, AUCBanditMetaTechnique):
+      self.session.flush()
+      info = BanditInfo(tuning_run=self.tuning_run,
+                        c=self.root_technique.bandit.C,
+                        window=self.root_technique.bandit.window,)
+      self.session.add(info)
+      for t in self.root_technique.techniques:
+        subtechnique = BanditSubTechnique(bandit_info=info,
+                                          name=t.name)
+        self.session.add(subtechnique)
+
+
     self.objective.set_driver(self)
     self.pending_config_ids = set()
     self.best_result = None
+    self.new_results = []
 
     for t in self.plugins:
       t.set_driver(self)
@@ -86,8 +118,14 @@ class SearchDriver(DriverBase):
         elapsed = elapsed.total_seconds()
       except:  # python 2.6
         elapsed = elapsed.days * 86400 + elapsed.seconds
-      return elapsed > self.args.stop_after
-    return self.test_count > self.args.test_limit
+      if elapsed > self.args.stop_after:
+          return True
+    if self.args.test_limit and self.test_count > self.args.test_limit:
+        return True    
+    if self.extra_criteria:
+        if self.extra_criteria(self.new_results):
+            return True
+    return False
 
   def register_result_callback(self, desired_result, callback):
     if desired_result.result is not None:
@@ -122,7 +160,7 @@ class SearchDriver(DriverBase):
   def run_generation_techniques(self):
     tests_this_generation = 0
     self.plugin_proxy.before_techniques()
-    for z in xrange(self.args.parallelism):
+    for z in range(self.args.parallelism):
       if self.seed_cfgs:
         config = self.get_configuration(self.seed_cfgs.pop())
         dr = DesiredResult(configuration=config,
@@ -132,7 +170,7 @@ class SearchDriver(DriverBase):
                            tuning_run=self.tuning_run)
       else:
         dr = self.root_technique.desired_result()
-      if dr is None:
+      if dr is None or dr is False:
         log.debug("no desired result, skipping to testing phase")
         break
       self.session.flush()  # populate configuration_id
@@ -169,10 +207,12 @@ class SearchDriver(DriverBase):
     return tests_this_generation
 
   def process_new_results(self):
+    self.new_results = []
     for result in (self.results_query()
                        .filter_by(was_new_best=None)
                        .order_by(Result.collection_date)):
       self.plugin_proxy.on_result(result)
+      self.new_results.append(result)
       if self.best_result is None:
         self.best_result = result
         result.was_new_best = True
@@ -204,7 +244,7 @@ class SearchDriver(DriverBase):
           rv = []
           for plugin in plugins:
             rv.append(getattr(plugin, method_name)(*args, **kwargs))
-          return filter(lambda x: x is not None, rv)
+          return [x for x in rv if x is not None]
 
         return plugin_method_proxy
 
@@ -224,7 +264,7 @@ class SearchDriver(DriverBase):
     no_tests_generations = 0
 
     # prime pipeline with tests
-    for z in xrange(self.args.pipelining):
+    for z in range(self.args.pipelining):
       self.run_generation_techniques()
       self.generation += 1
 
