@@ -1,3 +1,6 @@
+from __future__ import print_function
+from builtins import map
+from builtins import object
 import abc
 import argparse
 import logging
@@ -5,6 +8,7 @@ import time
 
 from datetime import datetime
 from fn import _
+from future.utils import with_metaclass
 
 log = logging.getLogger(__name__)
 display_log = logging.getLogger(__name__ + ".DisplayPlugin")
@@ -57,8 +61,7 @@ class SearchPlugin(object):
     """
     pass
 
-class DisplayPlugin(SearchPlugin):
-  __metaclass__ = abc.ABCMeta
+class DisplayPlugin(with_metaclass(abc.ABCMeta, SearchPlugin)):
   def __init__(self, display_period=5):
     super(DisplayPlugin, self).__init__()
     self.last  = time.time()
@@ -117,14 +120,12 @@ class FileDisplayPlugin(SearchPlugin):
   def on_result(self, result):
     if self.out and result.time < self.last_best:
       self.last_best = result.time
-      print >>self.out, \
-          (result.collection_date - self.start_date).total_seconds(), \
-          result.time
+      print((result.collection_date - self.start_date).total_seconds(), \
+          result.time, file=self.out)
       self.out.flush()
     if self.details:
-      print >>self.details, \
-          (result.collection_date - self.start_date).total_seconds(), \
-          result.time
+      print((result.collection_date - self.start_date).total_seconds(), \
+          result.time, file=self.details)
       self.details.flush()
 
 def get_enabled(args):
