@@ -1,4 +1,6 @@
+from __future__ import print_function
 # vim: tabstop=2 shiftwidth=2 softtabstop=2 expandtab autoindent smarttab
+from builtins import object
 import argparse
 import copy
 import inspect
@@ -92,7 +94,7 @@ class TuningRunMain(object):
 
     manipulator = measurement_interface.manipulator()
     if args.print_search_space_size:
-      print "10^{%.2f}" % math.log(manipulator.search_space_size(), 10)
+      print("10^{%.2f}" % math.log(manipulator.search_space_size(), 10))
       sys.exit(0)
     # show internal parameter representation
     if args.print_params:
@@ -107,8 +109,8 @@ class TuningRunMain(object):
         else:
           params_dict[cls] = [p]
       for k in params_dict:
-        print k, params_dict[k]
-        print
+        print(k, params_dict[k])
+        print()
       sys.exit(0)
 
     input_manager = measurement_interface.input_manager()
@@ -171,6 +173,7 @@ class TuningRunMain(object):
         'tuning_run_main': self,
         'tuning_run': self.tuning_run,
         'extra_seeds': self.measurement_interface.seed_configurations(),
+        'extra_criteria': self.measurement_interface.extra_convergence_criteria
       }
 
       self.search_driver = self.search_driver_cls(**driver_kwargs)
@@ -196,8 +199,9 @@ class TuningRunMain(object):
       self.tuning_run.state = 'RUNNING'
       self.commit(force=True)
       self.search_driver.main()
-      self.measurement_interface.save_final_config(
-          self.search_driver.best_result.configuration)
+      if self.search_driver.best_result:
+        self.measurement_interface.save_final_config(
+            self.search_driver.best_result.configuration)
       self.tuning_run.final_config = self.search_driver.best_result.configuration
       self.tuning_run.state = 'COMPLETE'
     except:
@@ -211,8 +215,9 @@ class TuningRunMain(object):
   def results_wait(self, generation):
     """called by search_driver to wait for results"""
     #single process version:
+    self.measurement_interface.pre_process()  
     self.measurement_driver.process_all()
-
+    self.measurement_interface.post_process()
 
 def main(interface, args, *pargs, **kwargs):
   if inspect.isclass(interface):
