@@ -1,4 +1,5 @@
 
+from past.builtins import execfile
 import sys
 from os.path import normpath, realpath, dirname, join, isfile
 
@@ -9,8 +10,5 @@ if 'venv' not in ','.join(sys.path):
   if isfile(venv_activate):
     execfile(venv_activate, dict(__file__=venv_activate))
 
-try:
-  import opentuner
-except:
-  sys.path.append(project_root)
+sys.path.insert(0, project_root)
 
