@@ -1,7 +1,8 @@
 #!/usr/bin/env python
 
+from __future__ import absolute_import
 if __name__ == '__main__':
-  import adddeps
+  from . import adddeps
 
 import argparse
 import logging
